#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import gettext
import json
import sys
import requests
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.gui import *
from ikabot.config import *

t = gettext.translation('insertCookies',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def importExportCookie(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	banner()
	try:
		print('Do you want to import or export the cookie?')
		print('(0) Exit')
		print('(1) Import')
		print('(2) Export')
		action = read(min=0, max=2)
		if action == 1:
			importCookie(session)
		elif action == 2:
			exportCookie(session)
		
		event.set()
	except KeyboardInterrupt:
		event.set()
		return

def importCookie(session):
	banner()
	print('{}⚠️ INSERTING AN INVALID COOKIE WILL LOG YOU OUT OF YOUR OTHER SESSIONS ⚠️{}\n\n'.format(bcolors.WARNING, bcolors.ENDC))
	print('Go ahead and export the cookie from another ikabot instance now and then')
	print('type your "ikariam" cookie below:')
	newcookie = read()
	newcookie = newcookie.strip()
	newcookie = newcookie.replace('ikariam=','')
	cookies = session.getSessionData()['cookies']
	cookies['ikariam'] = newcookie
	if session.host in session.s.cookies._cookies:
		session.s.cookies.set('ikariam', newcookie, domain = session.host, path = '/')
	else:
		session.s.cookies.set('ikariam', newcookie, domain = '', path = '/')

	html = session.s.get(session.urlBase).text

	if session.isExpired(html):
		print('{}Failure!{} All your other sessions have just been invalidated!'.format(bcolors.RED, bcolors.ENDC))
		enter()
	else:
		print('{}Success!{} This ikabot session will now use the cookie you provided'.format(bcolors.GREEN, bcolors.ENDC))
		sessionData = session.getSessionData()
		sessionData['cookies']['ikariam'] = newcookie
		session.setSessionData(sessionData)
		enter()
	session.get()

def exportCookie(session):
	banner()
	session.get() #get valid cookie in case user has logged the bot out before running this feature
	ikariam = session.getSessionData()['cookies']['ikariam']
	print('Use this cookie to synchronise two ikabot instances on 2 different machines\n\n')
	print('ikariam='+ikariam+'\n\n')

	cookie = json.dumps({"ikariam" : ikariam}) #get ikariam cookie, only this cookie is invalidated when the bot logs the user out.
	cookies_js = 'cookies={};i=0;for(let cookie in cookies){{document.cookie=Object.keys(cookies)[i]+\"=\"+cookies[cookie];i++}}'.format(cookie)
	print("""To prevent ikabot from logging you out while playing Ikariam do the following:
	1. Be on the "Your session has expired" screen
	2. Open Chrome javascript console by pressing CTRL + SHIFT + J
	3. Copy the text below, paste it into the console and press enter
	4. Press F5
	""")
	print(cookies_js)
	enter()
	


#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import signal
import gettext
from ikabot.config import *
from ikabot.helpers.botComm import *

t = gettext.translation('signals',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def do_nothing(signal, frame):
	pass

def deactivate_sigint():
	signal.signal(signal.SIGINT, do_nothing) #signal.SIGHUP replaced with signal.SIGINT for compatibility

def create_handler(s):
	def _handler(signum, frame):
		raise Exception(_('Signal number {:d} received').format(signum))
	return _handler

def setSignalsHandlers(s):
	signals = [signal.SIGINT, signal.SIGTERM] #signal.SIGQUIT replaced with signal.SIGINT for compatibility
	for sgn in signals:
		signal.signal(sgn, create_handler(s))

def setInfoSignal(session, info): # send process info to bot
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	info : str
	"""
	info = _('information of the process {}:\n{}').format(os.getpid(), info)
	def _sendInfo(signum, frame):
		sendToBot(session, info)
	signal.signal(signal.SIGABRT, _sendInfo) # kill -SIGUSR1 pid, SIGUSR1 replaced with SIGABRT for compatibility



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import re
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.naval import *
from ikabot.helpers.varios import *
from ikabot.helpers.resources import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import getCity
from ikabot.helpers.market import getGold

t = gettext.translation('getStatus',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def getStatus(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()
		color_arr = [bcolors.ENDC, bcolors.HEADER, bcolors.STONE, bcolors.BLUE, bcolors.WARNING]

		(ids, __) = getIdsOfCities(session)
		total_resources = [0] * len(materials_names)
		total_production = [0] * len(materials_names)
		total_wine_consumption = 0
		available_ships = 0
		total_ships = 0
		for id in ids:
			session.get('view=city&cityId={}'.format(id), noIndex = True)
			data = session.get("view=updateGlobalData&ajax=1", noIndex = True)
			json_data = json.loads(data, strict=False)
			json_data = json_data[0][1]['headerData']
			if json_data['relatedCity']['owncity'] != 1:
				continue
			wood = Decimal( json_data['resourceProduction'] )
			good = Decimal( json_data['tradegoodProduction'] )
			typeGood = int( json_data['producedTradegood'] )
			total_production[0] += wood * 3600
			total_production[typeGood] += good * 3600
			total_wine_consumption += json_data['wineSpendings']
			total_resources[0] += json_data['currentResources']['resource']
			total_resources[1] += json_data['currentResources']['1']
			total_resources[2] += json_data['currentResources']['2']
			total_resources[3] += json_data['currentResources']['3']
			total_resources[4] += json_data['currentResources']['4']
			available_ships = json_data['freeTransporters']
			total_ships = json_data['maxTransporters']
			total_gold = int(Decimal(json_data['gold']))
			total_gold_production = int(Decimal(json_data['scientistsUpkeep'] + json_data['income'] + json_data['upkeep']))
		print(_('Ships {:d}/{:d}').format(int(available_ships), int(total_ships)))
		print(_("\nTotal:"))
		print('{:>10}'.format(' '), end='|')
		for i in range(len(materials_names)):
			print('{:>12}'.format(materials_names_english[i]), end = '|')
		print()
		print('{:>10}'.format('Available'), end = '|')
		for i in range(len(materials_names)):
			print('{:>12}'.format(addThousandSeparator(total_resources[i], ' ')), end = '|')
		print()
		print('{:>10}'.format('Production'), end = '|')
		for i in range(len(materials_names)):
			print('{:>12}'.format(addThousandSeparator(total_production[i], ' ')), end = '|')
		print()
		print("Gold : {}, Gold production : {}".format(addThousandSeparator(total_gold, ' '), addThousandSeparator(total_gold_production, ' ')))
		print("Wine consumption : {}".format(addThousandSeparator(total_wine_consumption, ' ')), end='')


		print(_('\nOf which city do you want to see the state?'))
		city = chooseCity(session)
		banner()

		(wood, good, typeGood) = getProductionPerSecond(session, city['id'])
		print('\033[1m{}{}{}'.format(color_arr[int(typeGood)], city['cityName'], color_arr[0]))

		resources = city['recursos']
		storageCapacity = city['storageCapacity']
		color_resources = []
		for i in range(len(materials_names)):
			if resources[i] == storageCapacity:
				color_resources.append(bcolors.RED)
			else:
				color_resources.append(bcolors.ENDC)
		print(_('Storage:'))
		print(addThousandSeparator(storageCapacity))
		print(_('Resources:'))
		for i in range(len(materials_names)):
			print('{} {}{}{} '.format(materials_names[i], color_resources[i], addThousandSeparator(resources[i]), bcolors.ENDC), end='')
		print('')

		print(_('Production:'))
		print('{}:{} {}:{}'.format(materials_names[0], addThousandSeparator(wood*3600), materials_names[typeGood], addThousandSeparator(good*3600)))

		hasTavern = 'tavern' in [ building['building'] for building in city['position'] ]
		if hasTavern:
			consumption_per_hour = city['consumo']
			if consumption_per_hour == 0:
				print(_('{}{}Does not consume wine!{}').format(bcolors.RED, bcolors.BOLD, bcolors.ENDC))
			else:
				if typeGood == 1 and (good*3600) > consumption_per_hour:
					elapsed_time_run_out = '∞'
				else:
					consumption_per_second = Decimal(consumption_per_hour) / Decimal(3600)
					remaining_resources_to_consume = Decimal(resources[1]) / Decimal(consumption_per_second)
					elapsed_time_run_out = daysHoursMinutes(remaining_resources_to_consume)
				print(_('There is wine for: {}').format(elapsed_time_run_out))

		for building in [ building for building in city['position'] if building['name'] != 'empty' ]:
			if building['isMaxLevel'] is True:
				color = bcolors.BLACK
			elif building['canUpgrade'] is True:
				color = bcolors.GREEN
			else:
				color = bcolors.RED

			level = building['level']
			if level < 10:
				level = ' ' + str(level)
			else:
				level = str(level)
			if building['isBusy'] is True:
				level = level + '+'

			print(_('lv:{}\t{}{}{}').format(level, color, building['name'], bcolors.ENDC))

		enter()
		print('')
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCity
from ikabot.helpers.planRoutes import executeRoutes
from ikabot.helpers.resources import *
from ikabot.helpers.varios import addThousandSeparator
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.gui import banner

t = gettext.translation('distributeResources',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def distributeResources(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()

		print(_('What resource do you want to distribute?'))
		print(_('(0) Exit'))
		for i in range(len(materials_names)):
			print('({:d}) {}'.format(i+1, materials_names[i]))
		resource = read(min=0, max=5)
		if resource == 0:
			event.set() #give main process control before exiting
			return
		resource -= 1

		if resource == 0:
			evenly = True
		else:
			print('\nHow do you want to distribute the resources?')
			print('1) From cities that produce them to cities that do not')
			print('2) Distribute them evenly among all cities')
			type_distribution = read(min=1, max=2)
			evenly = type_distribution == 2

		if evenly:
			routes = distribute_evenly(session, resource)
		else:
			routes = distribute_unevenly(session, resource)

		if routes is None:
			event.set()
			return

		banner()
		print(_('\nThe following shipments will be made:\n'))
		for route in routes:
			print('{} -> {} : {} {}'.format(route[0]['name'], route[1]['name'], route[resource+3], materials_names[resource])) #displays all routes to be executed in console

		print(_('\nProceed? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			event.set()
			return

	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set() #this is where we give back control to main process

	info = _('\nDistribute {}\n').format(materials_names[resource])
	setInfoSignal(session, info)

	try:
		executeRoutes(session, routes) #plan trips for all the routes
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg) #sends message to telegram bot
	finally:
		session.logout()

def distribute_evenly(session, resource_type):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	resource_type : int
	"""
	resourceTotal = 0
	(cityIDs, cities) = getIdsOfCities(session)

	originCities = {}
	destinationCities = {}
	allCities = {}
	for cityID in cityIDs:

		html = session.get(city_url + cityID) #load html from the get request for that particular city
		city = getCity(html) #convert the html to a city object

		resourceTotal += city['recursos'][resource_type] #the cities resources are added to the total
		allCities[cityID] = city #adds the city to all cities


	# if a city doesn't have enough storage to fit resourceAverage
	# ikabot will send enough resources to fill the store to the max
	# then, resourceAverage will be recalculated
	resourceAverage = resourceTotal // len(allCities)
	while True:

		len_prev = len(destinationCities)
		for cityID in allCities:
			if cityID in destinationCities:
				continue
			freeStorage = allCities[cityID]['freeSpaceForResources'][resource_type]
			storage = allCities[cityID]['storageCapacity']
			if storage < resourceAverage:
				destinationCities[cityID] = freeStorage
				resourceTotal -= storage

		resourceAverage = resourceTotal // ( len(allCities) - len(destinationCities) )

		if len_prev == len(destinationCities):
			for cityID in allCities:
				if cityID in destinationCities:
					continue
				if allCities[cityID]['recursos'][resource_type] > resourceAverage:
					originCities[cityID] = allCities[cityID]['recursos'][resource_type] - resourceAverage
				else:
					destinationCities[cityID] = resourceAverage - allCities[cityID]['recursos'][resource_type]
			break

	originCities = {k: v for k, v in sorted(originCities.items(), key=lambda item: item[1],reverse=True)} #sort origin cities in descending order
	destinationCities = {k: v for k, v in sorted(destinationCities.items(), key=lambda item: item[1])}    #sort destination cities in ascending order

	routes = []

	for originCityID in originCities: #iterate through all origin city ids

		for destinationCityID in destinationCities: #iterate through all destination city ids
			if originCities[originCityID] == 0 or destinationCities[destinationCityID] == 0:
				continue

			if originCities[originCityID] > destinationCities[destinationCityID]: #if there's more resources above average in the origin city than resources below average in the destination city (origin city needs to have a surplus and destination city needs to have a deficit of resources for a route to be considered)
				toSend = destinationCities[destinationCityID] #number of resources to send is the number of resources below average in destination city
			else:
				toSend = originCities[originCityID] #send the amount of resources above average of the current origin city

			if toSend == 0:
				continue

			toSendArr = [0] * len(materials_names)
			toSendArr[resource_type] = toSend
			route = (allCities[originCityID], allCities[destinationCityID], allCities[destinationCityID]['islandId'], *toSendArr)
			routes.append(route)

			# ROUTE BLOCK
			if originCities[originCityID] > destinationCities[destinationCityID]:
				originCities[originCityID] -= destinationCities[destinationCityID] #remove the sent amount from the origin city's surplus
				destinationCities[destinationCityID] = 0 #set the amount of resources below average in destination city to 0
			else:
				destinationCities[destinationCityID] -= originCities[originCityID] #remove the sent amount from the amount of resources below average in current destination city
				originCities[originCityID] = 0 #set the amount of resources above average in origin city to 0

	return routes

def distribute_unevenly(session, resource_type):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	resource_type : int
	"""
	total_available_resources_from_all_cities = 0
	(cities_ids, cities) = getIdsOfCities(session)
	origin_cities = {}
	destination_cities = {}
	for destination_city_id in cities_ids:
		is_city_mining_this_resource = cities[destination_city_id]['tradegood'] == str(resource_type)
		if is_city_mining_this_resource:
			html = session.get(city_url + destination_city_id)
			city = getCity(html)
			if resource_type == 1: # wine
				city['available_amount_of_resource'] = city['recursos'][resource_type] - city['consumo'] - 1
			else:
				city['available_amount_of_resource'] = city['recursos'][resource_type]
			if city['available_amount_of_resource'] < 0:
				city['available_amount_of_resource'] = 0
			total_available_resources_from_all_cities += city['available_amount_of_resource']
			origin_cities[destination_city_id] = city
		else:
			html = session.get(city_url + destination_city_id)
			city = getCity(html)
			city['free_storage_for_resource'] = city['freeSpaceForResources'][resource_type]
			if city['free_storage_for_resource'] > 0:
				destination_cities[destination_city_id] = city

	if total_available_resources_from_all_cities <= 0:
		print(_('\nThere are no resources to send.'))
		enter()
		return None
	if len(destination_cities) == 0:
		print(_('\nThere is no space available to send resources.'))
		enter()
		return None

	remaining_resources_to_be_sent_to_each_city = total_available_resources_from_all_cities // len(destination_cities)
	free_storage_available_per_city = [destination_cities[city]['free_storage_for_resource'] for city in destination_cities]
	total_free_storage_available_in_all_cities = sum(free_storage_available_per_city)
	remaining_resources_to_send = min(total_available_resources_from_all_cities, total_free_storage_available_in_all_cities)
	toSend = {}

	while remaining_resources_to_send > 0:
		len_prev = len(toSend)
		for city_id in destination_cities:
			city = destination_cities[city_id]
			if city_id not in toSend and city['free_storage_for_resource'] < remaining_resources_to_be_sent_to_each_city:
				toSend[city_id] = city['free_storage_for_resource']
				remaining_resources_to_send -= city['free_storage_for_resource']

		if len(toSend) == len_prev:
			for city_id in destination_cities:
				if city_id not in toSend:
					toSend[city_id] = remaining_resources_to_be_sent_to_each_city
			break

		free_storage_available_per_city = [ destination_cities[city]['free_storage_for_resource'] for city in destination_cities if city not in toSend ]
		if len(free_storage_available_per_city) == 0:
			break
		total_free_storage_available_in_all_cities = sum( free_storage_available_per_city )
		remaining_resources_to_send = min(remaining_resources_to_send, total_free_storage_available_in_all_cities)
		remaining_resources_to_be_sent_to_each_city = remaining_resources_to_send // len(free_storage_available_per_city)


	routes = []
	for destination_city_id in destination_cities:
		destination_city = destination_cities[destination_city_id]
		island_id = destination_city['islandId']
		missing_resources = toSend[destination_city_id]
		for origin_city_id in origin_cities:
			if missing_resources == 0:
				break

			origin_city = origin_cities[origin_city_id]
			resources_available_in_this_city = origin_city['available_amount_of_resource']
			for route in routes:
				origin = route[0]
				resource = route[resource_type + 3]
				if origin['id'] == origin_city_id:
					resources_available_in_this_city -= resource

			send_this_round = min(missing_resources, resources_available_in_this_city)
			available = destination_city['free_storage_for_resource']
			if available == 0 or send_this_round == 0:
				continue

			if available < send_this_round:
				missing_resources = 0
				send_this_round = available
			else:
				missing_resources -= send_this_round

			toSendArr = [0] * len(materials_names)
			toSendArr[resource_type] = send_this_round
			route = (origin_city, destination_city, island_id, *toSendArr)

			routes.append(route)

	return routes



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import enter, enter
from ikabot.helpers.varios import wait
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.pedirInfo import getIslandsIds
from ikabot.helpers.getJson import getIsland
from ikabot.helpers.process import set_child_mode

t = gettext.translation('searchForIslandSpaces',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def searchForIslandSpaces(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		if checkTelegramData(session) is False:
			event.set()
			return
		banner()
		print('Do you want to search for spaces on your islands or a specific set of islands?')
		print('(0) Exit')
		print('(1) Search all islands I have colonised')
		print('(2) Search a specific set of islands')
		choice = read(min=0, max = 2)
		islandList = []
		if choice == 0:
			event.set()
			return
		elif choice == 2:
			banner()
			print('Insert the coordinates of each island you want searched like so: X1:Y1, X2:Y2, X3:Y3...')
			coords_string = read()
			coords_string = coords_string.replace(' ', '')
			coords = coords_string.split(',')
			for coord in coords:
				coord = '&xcoord=' + coord
				coord = coord.replace(':', '&ycoord=')
				html = session.get('view=island' + coord)
				island = getIsland(html)
				islandList.append(island['id'])
		else:
			pass
		
		banner()
		print('How frequently should the islands be searched in minutes (minimum is 3)?')
		time = read(min = 3, digit = True)

		banner()
		print(_('I will search for changes in the selected islands'))
		enter()
	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI search for new spaces each hour\n')
	setInfoSignal(session, info)
	try:
		do_it(session, islandList, time)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def do_it(session, islandList, time):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	islandList : list[dict]
		A list containing island objects which should be searched, if an empty list is passed, all the user's colonised islands are searched
	time : int
		The time in minutes between two consecutive seraches
	"""

	# this dict will contain all the cities from each island
	# as they where in last scan
	cities_before_per_island = {}

	while True:
		# this is done inside the loop because the user may colonize in a new island
		if islandList != []:
			islandsIds = islandList
		else:
			islandsIds = getIslandsIds(session)
		for islandId in islandsIds:
			html = session.get(island_url + islandId)
			island = getIsland(html)
			# cities in the current island
			cities_now = [city_space for city_space in island['cities'] if city_space['type'] != 'empty'] #loads the islands non empty cities into ciudades

			# if we haven't scaned this island before,
			# save it and do nothing
			if islandId not in cities_before_per_island:
				cities_before_per_island[islandId] = cities_now.copy()
			else:
				cities_before = cities_before_per_island[islandId]

				# someone disappeared
				for city_before in cities_before:
					if city_before['id'] not in [ city_now['id'] for city_now in cities_now ]:
						# we didn't find the city_before in the cities_now
						msg = _('the city {} of the player {} disappeared in {} {}:{} {}').format(city_before['name'], city_before['Name'], materials_names[int(island['tradegood'])], island['x'], island['y'], island['name'])
						sendToBot(session, msg)
						cities_before_per_island[islandId] = cities_now.copy() #update cities_before_per_island for the current island

				# someone colonised
				for city_now in cities_now:
					if city_now['id'] not in [ city_before['id'] for city_before in cities_before ]:
						# we didn't find the city_now in the cities_before
						msg = _('{} founded {} in {} {}:{} {}').format(city_now['Name'], city_now['name'], materials_names[int(island['tradegood'])], island['x'], island['y'], island['name'])
						sendToBot(session, msg)
						cities_before_per_island[islandId] = cities_now.copy() #update cities_before_per_island for the current island

		wait(time * 60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import json
import math
import gettext
import traceback
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.naval import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planRoutes import waitForArrival

t = gettext.translation('attackBarbarians',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def choose_island(session):
	idsIslands = getIslandsIds(session)
	islands = []
	for idIsland in idsIslands:
		html = session.get(island_url + idIsland)
		island = getIsland(html)
		islands.append(island)

	if len(islands) == 1:
		return islands[0]

	islands.sort(key=lambda island:island['id'])

	longest_island_name_length = 0
	for island in islands:
		longest_island_name_length = max(len(island['name']), longest_island_name_length)

	pad = lambda island_name: ' ' * (longest_island_name_length - len(island_name)) + island_name

	print(_('In which island do you want to attack the barbarians?'))
	print(_(' 0) Exit'))
	for i, island in enumerate(islands):
		num = ' ' + str(i+1) if i < 9 else str(i+1)
		if island['barbarians']['destroyed'] == 1:
			warn = _('(currently destroyed)')
		else:
			warn = ''
		print(_('{}) [{}:{}] {} ({}) : barbarians lv: {} ({}) {}').format(num, island['x'], island['y'], pad(island['name']), materials_names[int(island['tradegood'])][0].upper(), island['barbarians']['level'], island['barbarians']['city'], warn))

	index = read(min=0, max=len(islands))
	if index == 0:
		return None
	else:
		return islands[index-1]

def get_barbarians_lv(session, island):
	params = {"view": "barbarianVillage", "destinationIslandId": island['id'], "oldBackgroundView": "city", "cityWorldviewScale": "1", "islandId": island['id'], "backgroundView": "island", "currentIslandId": island['id'], "actionRequest": actionRequest, "ajax": "1"}
	resp = session.post(params=params)
	resp = json.loads(resp, strict=False)

	level = int(resp[2][1]['js_islandBarbarianLevel']['text'])
	gold  = int(resp[2][1]['js_islandBarbarianResourcegold']['text'].replace(',', ''))

	resources = [0] * len(materials_names)
	for i in range(len(materials_names)):
		if i == 0:
			resources[i] = int(resp[2][1]['js_islandBarbarianResourceresource']['text'].replace(',', ''))
		else:
			resources[i] = int(resp[2][1]['js_islandBarbarianResourcetradegood{:d}'.format(i)]['text'].replace(',', ''))

	html = resp[1][1][1]
	troops = re.findall(r'<div class="army \w*?">\s*<div class=".*?">(.*?)</div>\s*</div>\s*</td>\s*</tr>\s*<tr>\s*<td class="center">\s*(\d+)', html)

	total_cargo = sum(resources)
	ships = math.ceil(Decimal(total_cargo) / Decimal(500))

	info = {
		'island_id': island['id'],
		'level': level,
		'gold': gold,
		'resources': resources,
		'troops': troops,
		'ships': ships
	}

	return info

def get_units(session, city):
	params = {
		'view': 'cityMilitary',
		'activeTab': 'tabUnits',
		'cityId': city['id'],
		'backgroundView': 'city',
		'currentCityId': city['id'],
		'currentTab': 'multiTab1',
		'actionRequest': actionRequest,
		'ajax': '1'
	}

	resp = session.post(params=params)
	resp = json.loads(resp, strict=False)
	html = resp[1][1][1]
	html = html.split('<div class="fleet')[0]

	unit_id_names   = re.findall(r'<div class="army (.*?)">\s*<div class="tooltip">(.*?)<\/div>', html)
	unit_amounts = re.findall(r'<td>(.*?)\s*</td>', html) # fixed lior 

	units = {}
	for i in range(len(unit_id_names)):
		amount = int(unit_amounts[i].replace(',', '').replace('-', '0'))
		unit_id = unit_id_names[i][0][1:]
		unit_name = unit_id_names[i][1]
		units[unit_id] = {}
		units[unit_id]['name'] = unit_name
		units[unit_id]['amount'] = amount

	return units

def plan_attack(session, city, babarians_info):
	total_units = get_units(session, city)

	if sum( [ total_units[unit_id]['amount'] for unit_id in total_units ] ) == 0:
		print('You don\'t have any troops in this city!')
		enter()
		return None

	plan = []
	total_ships = None
	last = False
	while True:

		banner()

		units_available = {}
		for unit_id in total_units:

			already_sent = sum( [ p['units'][u] for p in plan for u in p['units'] if u == unit_id ] )
			if already_sent < total_units[unit_id]['amount']:
				units_available[unit_id] = {}
				units_available[unit_id]['amount'] = total_units[unit_id]['amount'] - already_sent
				units_available[unit_id]['name']   = total_units[unit_id]['name']

		if len(units_available) == 0:
			print(_('No more troops available to send'))
			enter()
			break

		attack_round = {}
		attack_round['units'] = {}
		print(_('Which troops do you want to send?').format(len(plan)+1))
		for unit_id in units_available:
			unit_amount = units_available[unit_id]['amount']
			unit_name   = units_available[unit_id]['name']
			amount_to_send = read(msg='{} (max: {}): '.format(unit_name, addThousandSeparator(unit_amount)), max=unit_amount, default=0)
			if amount_to_send > 0:
				attack_round['units'][unit_id] = amount_to_send
		print('')

		attack_round['loot'] = last
		if last:
			attack_round['round'] = len(plan) + 1
		else:
			if len(plan) > 0:
				round_def = len(plan) + 1
				attack_round['round'] = read(msg=_('In which battle round do you want to send them? (min: 2, default: {:d}): ').format(round_def), min=2, default=round_def)
			else:
				attack_round['round'] = 1
		print('')

		#max_ships = babarians_info['ships']
		if last is False:
			if total_ships is None:
				total_ships = getTotalShips(session)
			max_ships = total_ships - sum( [ ar['ships'] for ar in plan ] )
			if max_ships > 0:
				attack_round['ships'] = read(msg=_('How many ships do you want to send in this round? (min: 0, max: {:d}): ').format(max_ships), min=0, max=max_ships)
				print('')
			else:
				attack_round['ships'] = 0

		plan.append(attack_round)

		if last:
			break

		print(_('Do you want to send another round of troops? [y/N]'))
		resp = read(values=['y', 'Y', 'n', 'N'], default='n')
		if resp.lower() != 'y':
			print('')
			print(_('Do you want to select the troops that will be used to collect the remaining resources? (they need to destroy the wall) [y/N]'))
			resp = read(values=['y', 'Y', 'n', 'N'], default='n')
			if resp.lower() != 'y':
				break
			else:
				last = True


	plan.sort(key=lambda ar:ar['round'])
	return plan

def attackBarbarians(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()

		island = choose_island(session)
		if island is None:
			event.set()
			return

		babarians_info = get_barbarians_lv(session, island)

		banner()
		print(_('The barbarians have:'))
		for name, amount in babarians_info['troops']:
			print(_('{} units of {}').format(amount, name))
		print('')

		banner()
		print(_('From which city do you want to attack?'))
		city = chooseCity(session)

		plan = plan_attack(session, city, babarians_info)
		if plan is None:
			event.set()
			return

		banner()
		print(_('The barbarians in [{}:{}] will be attacked.').format(island['x'], island['y']))
		enter()

	except KeyboardInterrupt:
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI attack the barbarians in [{}:{}]\n').format(island['x'], island['y'])
	setInfoSignal(session, info)
	try:
		do_it(session, island, city, babarians_info, plan)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def get_unit_data(session, city_id, unit_id):
	params_w = {
		'view': 'unitdescription',
		'unitId': unit_id,
		'helpId': 9,
		'subHelpId': 0,
		'backgroundView': 'city',
		'currentCityId': city_id,
		'templateView': 'unitdescription',
		'actionRequest': actionRequest,
		'ajax': 1
	}
	resp = session.post(params=params_w)
	resp = json.loads(resp, strict=False)
	html = resp[1][1][1]

	weight = re.search(r'<li class="weight fifthpos" title=".*?"><span\s*class="accesshint">\'.*?\': </span>(\d+)</li>', html).group(1)
	weight = int(weight)

	speed = re.search(r'(\d+)\s*<br\/>\s*<span class="textLabel">.*?\s*:<\/span>\d+<br\/>\s*<\/div>\s*<div class="clearfloat"><\/div>\s*<div class="weapon">', html).group(1)
	speed = int(speed)

	return {'speed': speed, 'weight': weight}

def city_is_in_island(city, island):
	return city['x'] == island['x'] and city['y'] == island['y']

def get_barbarians_info(session, island_id):
	query = {
		'view': 'barbarianVillage',
		'destinationIslandId': island_id,
		'backgroundView': 'island',
		'currentIslandId': island_id,
		'actionRequest': actionRequest,
		'ajax': 1
	}
	resp = session.post(params=query)
	resp = json.loads(resp, strict=False)
	return resp

def wait_until_can_attack(session, city, island, travel_time=0):
	html = session.get(island_url + island['id'])
	island = getIsland(html)

	if island['barbarians']['underAttack'] == 0 and island['barbarians']['destroyed'] == 0:
		# an attack might be on its way
		wait_for_arrival(session, city, island)
		html = session.get(island_url + island['id'])
		island = getIsland(html)

	if island['barbarians']['underAttack'] == 1:
		# a battle is taking place
		attacks = get_current_attacks(session, city['id'], island['id'])
		attacks_fighting = filter_fighting(attacks)
		eventTimes = [ attack['eventTime'] for attack in attacks_fighting ]
		if len(eventTimes) > 0:
			wait_time  = max(eventTimes)
			wait_time -= time.time()
			wait(wait_time + 5)
		wait_until_can_attack(session, city, island, travel_time)

	if island['barbarians']['destroyed'] == 1:
		# the barbarians are destroyed and can't be attacked
		resp = get_barbarians_info(session, island['id'])
		if 'barbarianCityCooldownTimer' in resp[2][1]:
			wait_time  = resp[2][1]['barbarianCityCooldownTimer']['countdown']['enddate']
			wait_time -= time.time()
			wait_time -= travel_time
			wait(wait_time + 5)
		wait_until_can_attack(session, city, island, travel_time)

def get_movements(session, city_id=None):
	if city_id is None:
		city_id = getCurrentCityId(session)
	query = {
		'view': 'militaryAdvisor',
		'oldView': 'updateGlobalData',
		'cityId': city_id,
		'backgroundView': 'city',
		'currentCityId': city_id,
		'templateView': 'militaryAdvisor',
		'actionRequest': actionRequest,
		'ajax': 1
	}

	resp = session.post(params=query)
	resp = json.loads(resp, strict=False)
	movements = resp[1][1][2]['viewScriptParams']['militaryAndFleetMovements']

	return movements

def get_current_attacks(session, city_id, island_id):

	movements = get_movements(session, city_id)
	curr_attacks = []

	for movement in movements:
		if movement['event']['mission'] != 13:
			continue
		if movement['target']['islandId'] != int(island_id):
			continue
		if movement['event']['isReturning'] != 0:
			continue
		if movement['origin']['cityId'] == -1:
			continue

		curr_attacks.append(movement)

	return curr_attacks

def wait_for_arrival(session, city, island):
	attacks = get_current_attacks(session, city['id'], island['id'])
	attacks = filter_loading(attacks) + filter_traveling(attacks)
	eventTimes = [ attack['eventTime'] for attack in attacks ]

	if len(eventTimes) == 0:
		return

	wait_time  = max(eventTimes)
	wait_time -= time.time()
	wait(wait_time + 5)

	wait_for_arrival(session, city, island)

def wait_for_round(session, city, island, travel_time, battle_start, round_number):
	if round_number == 1:
		wait_until_can_attack(session, city, island, travel_time)
	else:
		wait_time  = battle_start + (round_number - 2) * 15 * 60
		wait_time -= time.time()
		wait_time -= travel_time
		wait(wait_time + 5)

		if battle_start < time.time():
			html = session.get(island_url + city['id'])
			island = getIsland(html)
			assert island['barbarians']['underAttack'] == 1, "the battle ended before expected"

def calc_travel_time(city, island, speed):
	if city['x'] == island['x'] and city['y'] == island['y']:
		return math.ceil(36000/speed)
	else:
		return math.ceil(1200 * math.sqrt( ((city['x'] - island['x']) ** 2) + ((city['y'] - island['y']) ** 2) )) # lior fix

def filter_loading(attacks):
	return [ attack for attack in attacks if attack['event']['missionState'] == 1 ]

def filter_traveling(attacks):
	return [ attack for attack in attacks if attack['event']['missionState'] == 2 and attack['event']['canAbort'] ]

def filter_fighting(attacks):
	return [ attack for attack in attacks if attack['event']['missionState'] == 2 and attack['event']['canRetreat'] ]

def wait_until_attack_is_over(session, city, island):
	html = session.get(island_url + island['id'])
	island = getIsland(html)

	while island['barbarians']['destroyed'] == 0:

		attacks = get_current_attacks(session, city['id'], island['id'])
		# the attack probably failed
		if len(attacks) == 0:
			return

		eventTimes = [ attack['eventTime'] for attack in attacks ]
		wait_time  = min(eventTimes)
		wait_time -= time.time()
		wait(wait_time + 5)

		html = session.get(island_url + island['id'])
		island = getIsland(html)

def load_troops(session, city, island, attack_round, units_data, attack_data, extra_cargo=0):
	ships_needed = Decimal(extra_cargo) / Decimal(500)
	speeds = []
	current_units = get_units(session, city)
	for unit_id in attack_round['units']:
		amount_to_send = min(attack_round['units'][unit_id], current_units[unit_id]['amount'])
		attack_data['cargo_army_{}'.format(unit_id)] = amount_to_send

		if unit_id not in units_data:
			units_data[unit_id] = get_unit_data(session, city['id'], unit_id)

		speeds.append(units_data[unit_id]['speed'])

		if city_is_in_island(city, island) is False:
			weight = units_data[unit_id]['weight']
			ships_needed += Decimal(amount_to_send * weight) / Decimal(500)

	ships_needed = math.ceil(ships_needed)
	speed = min(speeds)
	travel_time = calc_travel_time(city, island, speed)
	return attack_data, ships_needed, travel_time

def loot(session, island, city, units_data, loot_round):
	while True:

		attack_data = {'action': 'transportOperations', 'function': 'attackBarbarianVillage', 'actionRequest': actionRequest, 'islandId': island['id'], 'destinationCityId': 0, 'cargo_army_304_upkeep': 3, 'cargo_army_304': 0, 'cargo_army_315_upkeep': 1, 'cargo_army_315': 0, 'cargo_army_302_upkeep': 4, 'cargo_army_302': 0, 'cargo_army_303_upkeep': 3, 'cargo_army_303': 0, 'cargo_army_312_upkeep': 15, 'cargo_army_312': 0, 'cargo_army_309_upkeep': 45, 'cargo_army_309': 0, 'cargo_army_307_upkeep': 15, 'cargo_army_307': 0, 'cargo_army_306_upkeep': 25, 'cargo_army_306': 0, 'cargo_army_305_upkeep': 30, 'cargo_army_305': 0, 'cargo_army_311_upkeep': 20, 'cargo_army_311': 0, 'cargo_army_310_upkeep': 10, 'cargo_army_310': 0, 'transporter': 0, 'barbarianVillage': 1, 'backgroundView': 'island', 'currentIslandId': island['id'], 'templateView': 'plunder', 'ajax': 1}

		# make sure we have ships on the port
		ships_available = waitForArrival(session)

		# if the barbarians are active again or all the resources were stolen, return
		html = session.get(island_url + island['id'])
		island = getIsland(html)
		destroyed = island['barbarians']['destroyed'] == 1
		resources = get_barbarians_lv(session, island)['resources']
		if destroyed is False or sum(resources) == 0:
			return

		# if we already sent an attack and we still have ships on the port, it was the last one
		attacks = get_current_attacks(session, city['id'], island['id'])
		attacks = filter_loading(attacks) + filter_traveling(attacks)
		if len(attacks) > 0:
			return

		attack_data, ships_needed, travel_time = load_troops(session, city, island, attack_round, units_data, attack_data, sum(resources))
		attack_data['transporter'] = min(ships_available, ships_needed)

		# make sure we have time to send the attack
		time_left = None
		resp = get_barbarians_info(session, island['id'])
		if 'barbarianCityCooldownTimer' in resp[2][1]:
			time_left  = resp[2][1]['barbarianCityCooldownTimer']['countdown']['enddate']
			time_left -= time.time()
		if time_left is not None and travel_time > time_left:
			return

		# send attack
		session.post(payloadPost=attack_data)

def do_it(session, island, city, babarians_info, plan):

	units_data = {}

	battle_start = None

	for attack_round in plan:

		# this round is supposed to get the resources
		if attack_round['loot']:
			break

		attack_data = {'action': 'transportOperations', 'function': 'attackBarbarianVillage', 'actionRequest': actionRequest, 'islandId': island['id'], 'destinationCityId': 0, 'cargo_army_304_upkeep': 3, 'cargo_army_304': 0, 'cargo_army_315_upkeep': 1, 'cargo_army_315': 0, 'cargo_army_302_upkeep': 4, 'cargo_army_302': 0, 'cargo_army_303_upkeep': 3, 'cargo_army_303': 0, 'cargo_army_312_upkeep': 15, 'cargo_army_312': 0, 'cargo_army_309_upkeep': 45, 'cargo_army_309': 0, 'cargo_army_307_upkeep': 15, 'cargo_army_307': 0, 'cargo_army_306_upkeep': 25, 'cargo_army_306': 0, 'cargo_army_305_upkeep': 30, 'cargo_army_305': 0, 'cargo_army_311_upkeep': 20, 'cargo_army_311': 0, 'cargo_army_310_upkeep': 10, 'cargo_army_310': 0, 'transporter': 0, 'barbarianVillage': 1, 'backgroundView': 'island', 'currentIslandId': island['id'], 'templateView': 'plunder', 'ajax': 1}

		attack_data, ships_needed, travel_time = load_troops(session, city, island, attack_round, units_data, attack_data)

		try:
			wait_for_round(session, city, island, travel_time, battle_start, attack_round['round'])
		except AssertionError:
			# battle ended before expected
			break

		ships_available = waitForArrival(session)
		while ships_available < ships_needed:
			ships_available = waitForArrival(session)
		ships_available -= ships_needed

		# if the number of available troops changed, the POST request might not work as intended

		attack_data['transporter'] = min(babarians_info['ships'], attack_round['ships'], ships_available)

		# send new round
		session.post(payloadPost=attack_data)

		if attack_round['round'] == 1:
			battle_start = time.time() + travel_time

	wait_until_attack_is_over(session, city, island)

	last_round = plan[-1]
	if last_round['loot']:
		loot(session, city, island, units_data, last_round)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from ikabot.config import *
from ikabot.helpers.getJson import getCity
from ikabot.helpers.pedirInfo import getIdsOfCities

def getCommercialCities(session):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session

	Returns
	-------
	commercial_cities : list[dict]
	"""
	cities_ids = getIdsOfCities(session)[0]
	commercial_cities = []
	for city_id in cities_ids:
		html = session.get(city_url + city_id)
		city = getCity(html)
		for pos, building in enumerate(city['position']):
			if building['building'] == 'branchOffice':
				city['pos'] = pos
				html = getMarketHtml(session, city)
				positions = re.findall(r'<option.*?>(\d+)</option>', html)
				city['rango'] = int(positions[-1])
				commercial_cities.append(city)
				break
	return commercial_cities

def getMarketHtml(session, city):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	"""
	url = 'view=branchOffice&cityId={}&position={:d}&currentCityId={}&backgroundView=city&actionRequest={}&ajax=1'.format(city['id'], city['pos'], city['id'], actionRequest)
	data = session.post(url)
	json_data = json.loads(data, strict=False)
	return json_data[1][1][1]

def storageCapacityOfMarket(html):
	match = re.search(r'var\s*storageCapacity\s*=\s*(\d+);', html)
	if match:
		return int(match.group(1))
	else:
		return 0

def onSellInMarket(html):
	mad, vin, mar, cri, azu = re.findall(r'<input type="text" class="textfield"\s*size="\d+"\s*name=".*?"\s*id=".*?"\s*value="(\d+)"', html)
	return [int(mad), int(vin), int(mar), int(cri), int(azu)]

def getGold(session, city):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict
	Returns
	-------
	gold : int
	"""
	url = 'view=finances&backgroundView=city&currentCityId={}&templateView=finances&actionRequest={}&ajax=1'.format(city['id'], actionRequest)
	data = session.post(url)
	json_data = json.loads(data, strict=False)
	gold = json_data[0][1]['headerData']['gold']
	gold = gold.split('.')[0]
	gold = int(gold)
	gold_production = json_data[0][1]['headerData']['scientistsUpkeep'] + json_data[0][1]['headerData']['income'] +json_data[0][1]['headerData']['upkeep']
	return gold, int(gold_production)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import math
import json
import gettext
import traceback
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.market import *
from ikabot.helpers.botComm import *
from ikabot.helpers.varios import addThousandSeparator, wait
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.planRoutes import waitForArrival

t = gettext.translation('sellResources',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def chooseCommercialCity(commercial_cities):
	"""
	Parameters
	----------
	commercial_cities : list[dict]

	Returns
	-------
	commercial_city : dict
	"""
	print(_('In which city do you want to sell resources?\n'))
	for i, city in enumerate(commercial_cities):
		print('({:d}) {}'.format(i + 1, city['name']))
	ind = read(min=1, max=len(commercial_cities))
	return commercial_cities[ind - 1]

def getMarketInfo(session, city):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city : dict

	Returns
	-------
	response : dict
	"""
	params = {'view': 'branchOfficeOwnOffers', 'activeTab': 'tab_branchOfficeOwnOffers', 'cityId': city['id'], 'position': city['pos'], 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': 'branchOfficeOwnOffers', 'currentTab': 'tab_branchOfficeOwnOffers', 'actionRequest': actionRequest, 'ajax': '1'}
	resp = session.post(params=params, noIndex=True)
	return json.loads(resp, strict=False)[1][1][1]

def getOffers(session, my_market_city, resource_type):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	my_market_city : dict
	resource_type : int

	Returns
	-------
	offers : list
	"""
	if resource_type == 0:
		resource_type = 'resource'
	else:
		resource_type = str(resource_type)
	data = {'cityId': my_market_city['id'], 'position': my_market_city['pos'], 'view': 'branchOffice', 'activeTab': 'bargain', 'type': '333', 'searchResource': resource_type, 'range': my_market_city['rango'], 'backgroundView': 'city', 'currentCityId': my_market_city['id'], 'templateView': 'branchOffice', 'currentTab': 'bargain', 'actionRequest': actionRequest, 'ajax': '1'}
	resp = session.post(payloadPost=data)
	html = json.loads(resp, strict=False)[1][1][1]
	return re.findall(r'<td class=".*?">(.*?)<br/>\((.*?)\)\s*</td>\s*<td>(.*?)</td>\s*<td><img src=".*?"\s*alt=".*?"\s*title=".*?"/></td>\s*<td style="white-space:nowrap;">(\d+)\s*<img src=".*?"\s*class=".*?"/>.*?</td>\s*<td>(\d+)</td>\s*<td><a onclick="ajaxHandlerCall\(this\.href\);return false;"\s*href="\?view=takeOffer&destinationCityId=(\d+)&', html)

def sellToOffers(session, city_to_buy_from, resource_type, event):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city_to_buy_from : dict
	resource_type : int
	event : multiprocessing.Event
	"""
	banner()

	offers = getOffers(session, city_to_buy_from, resource_type)

	if len(offers) == 0:
		print(_('No offers available.'))
		enter()
		event.set()
		return

	print(_('Which offers do you want to sell to?\n'))

	chosen_offers = []
	total_amount = 0
	profit = 0
	for offer in offers:
		cityname, username, amount, price, dist, destination_city_id = offer
		cityname = cityname.strip()
		amount = amount.replace(',', '').replace('.', '')
		amount = int(amount)
		price = int(price)
		msg = _('{} ({}): {} at {:d} each ({} in total) [Y/n]').format(cityname, username, addThousandSeparator(amount), price, addThousandSeparator(price*amount))
		rta = read(msg=msg, values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			continue
		chosen_offers.append(offer)
		total_amount += amount
		profit += amount * price

	if len(chosen_offers) == 0:
		event.set()
		return

	available = city_to_buy_from['recursos'][resource_type]
	amount_to_sell = min(available, total_amount)

	banner()
	print(_('\nHow much do you want to sell? [max = {}]').format(addThousandSeparator(amount_to_sell)))
	amount_to_sell = read(min=0, max=amount_to_sell)
	if amount_to_sell == 0:
		event.set()
		return

	left_to_sell = amount_to_sell
	profit = 0
	for offer in chosen_offers:
		cityname, username, amount, price, dist, destination_city_id = offer
		cityname = cityname.strip()
		amount = amount.replace(',', '').replace('.', '')
		amount = int(amount)
		price = int(price)
		sell = min(amount, left_to_sell)
		left_to_sell -= sell
		profit += sell * price
	print(_('\nSell {} of {} for a total of {}? [Y/n]').format(addThousandSeparator(amount_to_sell), materials_names[resource_type], addThousandSeparator(profit)))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI sell {} of {} in {}\n').format(addThousandSeparator(amount_to_sell), materials_names[resource_type], city_to_buy_from['name'])
	setInfoSignal(session, info)
	try:
		do_it1(session, amount_to_sell, chosen_offers, resource_type, city_to_buy_from)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def createOffer(session, my_offering_market_city, resource_type, event):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	my_offering_market_city : dict
	resource_type : int
	event : multiprocessing.Event
	"""
	banner()

	html = getMarketInfo(session, my_offering_market_city)
	sell_market_capacity = storageCapacityOfMarket(html)
	total_available_amount_of_resource = my_offering_market_city['recursos'][resource_type]

	print(_('How much do you want to sell? [max = {}]').format(addThousandSeparator(total_available_amount_of_resource)))
	amount_to_sell = read(min=0, max=total_available_amount_of_resource)
	if amount_to_sell == 0:
		event.set()
		return

	price_max, price_min = re.findall(r'\'upper\': (\d+),\s*\'lower\': (\d+)', html)[resource_type]
	price_max = int(price_max)
	price_min = int(price_min)
	print(_('\nAt what price? [min = {:d}, max = {:d}]').format(price_min, price_max))
	price = read(min=price_min, max=price_max)

	print(_('\nI will sell {} of {} at {}: {}').format(addThousandSeparator(amount_to_sell), materials_names[resource_type], addThousandSeparator(price), addThousandSeparator(price * amount_to_sell)))
	print(_('\nProceed? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		event.set()
		return

	set_child_mode(session)
	event.set()

	info = _('\nI sell {} of {} in {}\n').format(addThousandSeparator(amount_to_sell), materials_names[resource_type], my_offering_market_city['name'])
	setInfoSignal(session, info)
	try:
		do_it2(session, amount_to_sell, price, resource_type, sell_market_capacity, my_offering_market_city)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(session, msg)
	finally:
		session.logout()

def sellResources(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()

		commercial_cities = getCommercialCities(session)
		if len(commercial_cities) == 0:
			print(_('There is no store built'))
			enter()
			event.set()
			return

		if len(commercial_cities) == 1:
			city = commercial_cities[0]
		else:
			city = chooseCommercialCity(commercial_cities)
			banner()

		print(_('What resource do you want to sell?'))
		for index, material_name in enumerate(materials_names):
			print('({:d}) {}'.format(index+1, material_name))
		selected_material = read(min=1, max=len(materials_names))
		resource = selected_material - 1
		banner()

		print(_('Do you want to sell to existing offers (1) or do you want to make your own offer (2)?'))
		selected = read(min=1, max=2)
		[sellToOffers, createOffer][selected - 1](session, city, resource, event)
	except KeyboardInterrupt:
		event.set()
		return

def do_it1(session, left_to_sell, offers, resource_type, city_to_buy_from):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	left_to_sell : int
	offers : list[dict]
	resource_type : int
	city_to_buy_from : dict
	"""
	for offer in offers:
		cityname, username, amount, precio, dist, destination_city_id = offer
		cityname = cityname.strip()
		amount_to_buy = amount.replace(',', '').replace('.', '')
		amount_to_buy = int(amount_to_buy)
		while True:
			amount_to_sell = min(amount_to_buy, left_to_sell)
			ships_available = waitForArrival(session)
			ships_needed = math.ceil((Decimal(amount_to_sell) / Decimal(500)))
			ships_used = min(ships_available, ships_needed)
			if ships_needed > ships_used:
				amount_to_sell = ships_used * 500
			left_to_sell -= amount_to_sell
			amount_to_buy -= amount_to_sell

			data = {'action': 'transportOperations', 'function': 'sellGoodsAtAnotherBranchOffice', 'cityId': city_to_buy_from['id'], 'destinationCityId': destination_city_id, 'oldView': 'branchOffice', 'position': city_to_buy_from['pos'], 'avatar2Name': username, 'city2Name': cityname, 'type': '333', 'activeTab': 'bargain', 'transportDisplayPrice': '0', 'premiumTransporter': '0', 'capacity': '5', 'max_capacity': '5', 'jetPropulsion': '0', 'transporters': str(ships_used), 'backgroundView': 'city', 'currentCityId': city_to_buy_from['id'], 'templateView': 'takeOffer', 'currentTab': 'bargain', 'actionRequest': actionRequest, 'ajax': '1'}
			if resource_type == 0:
				data['cargo_resource'] = amount_to_sell
				data['resourcePrice'] = precio
			else:
				data['tradegood{:d}Price'.format(resource_type)] = precio
				data['cargo_tradegood{:d}'.format(resource_type)] = amount_to_sell

			session.get(city_url + city_to_buy_from['id'], noIndex=True)
			session.post(payloadPost=data)

			if left_to_sell == 0:
				return
			if amount_to_buy == 0:
				break

def do_it2(session, amount_to_sell, price, resource_type, sell_market_capacity, city):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	amount_to_sell : int
	price : int
	resource_type : int
	sell_market_capacity : int
	city : dict
	"""
	initial_amount_to_sell = amount_to_sell
	html = getMarketInfo(session, city)
	previous_on_sell = onSellInMarket(html)[resource_type]
	while True:
		html = getMarketInfo(session, city)
		currently_on_sell = onSellInMarket(html)[resource_type]
		# if there is space in the store
		if currently_on_sell < storageCapacityOfMarket(html):
			# add our new offer to the free space
			free_space = sell_market_capacity - currently_on_sell
			offer = min(amount_to_sell, free_space)
			amount_to_sell -= offer
			new_offer = currently_on_sell + offer

			payloadPost = {'cityId': city['id'], 'position': city['pos'], 'action': 'CityScreen', 'function': 'updateOffers', 'resourceTradeType': '444', 'resource': '0', 'resourcePrice': '10', 'tradegood1TradeType': '444', 'tradegood1': '0', 'tradegood1Price': '11', 'tradegood2TradeType': '444', 'tradegood2': '0', 'tradegood2Price': '12', 'tradegood3TradeType': '444', 'tradegood3': '0', 'tradegood3Price': '17', 'tradegood4TradeType': '444', 'tradegood4': '0', 'tradegood4Price': '5', 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': 'branchOfficeOwnOffers', 'currentTab': 'tab_branchOfficeOwnOffers', 'actionRequest': actionRequest, 'ajax': '1'}
			if resource_type == 0:
				payloadPost['resource'] = new_offer
				payloadPost['resourcePrice'] = price
			else:
				payloadPost['tradegood{:d}'.format(resource_type)] = new_offer
				payloadPost['tradegood{:d}Price'.format(resource_type)] = price
			session.post(payloadPost=payloadPost)

			# if we don't have any more to add to the offer, leave the loop
			if amount_to_sell == 0:
				break

		# sleep for 2 hours
		wait(60 * 60 *  2)

	# wait until the last of our offer is actualy bought, and let the user know
	while True:
		html = getMarketInfo(session, city)
		currently_on_sell = onSellInMarket(html)[resource_type]
		if currently_on_sell <= previous_on_sell:
			msg = _('{} of {} was sold at {:d}').format(addThousandSeparator(initial_amount_to_sell), materials_names[resource_type], price)
			sendToBot(session, msg)
			return

		# sleep for 2 hours
		wait(60 * 60 *  2)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import gettext
import requests
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.gui import *
import ikabot.config as config

t = gettext.translation('proxy',
                        config.localedir,
                        languages=config.languages,
                        fallback=True)
_ = t.gettext

def show_proxy(session):
	session_data = session.getSessionData()
	msg = _('using proxy:')
	if 'proxy' in session_data and session_data['proxy']['set'] is True:
		curr_proxy = session_data['proxy']['conf']['https']
		if test_proxy(session_data['proxy']['conf']) is False:
			session_data['proxy']['set'] = False
			session.setSessionData(session_data)
			sys.exit(_('the {} proxy does not work, it has been removed').format(curr_proxy))
		if msg not in config.update_msg:
			# add proxy message
			config.update_msg += '{} {}\n'.format(msg, curr_proxy)
		else:
			# delete old proxy message
			config.update_msg = config.update_msg.replace('\n'.join(config.update_msg.split('\n')[-2:]), '')
			# add new proxy message
			config.update_msg += '{} {}\n'.format(msg, curr_proxy)
	elif msg in config.update_msg:
		# delete old proxy message
		config.update_msg = config.update_msg.replace('\n'.join(config.update_msg.split('\n')[-2:]), '')

def test_proxy(proxy_dict):
	try:
		requests.get('https://lobby.ikariam.gameforge.com/', proxies=proxy_dict)	
	except:
		return False
	return True

def read_proxy():
	print(_('Enter the proxy (examples: socks5://127.0.0.1:9050, https://45.117.163.22:8080):'))
	proxy_str = read(msg='proxy: ')
	proxy_dict = {'http': proxy_str, 'https': proxy_str}
	if test_proxy(proxy_dict) is False:
		print(_('The proxy does not work.'))
		enter()
		return None
	print(_('The proxy works and it will be used for all future requests.'))
	enter()
	return proxy_dict

def proxyConf(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()
		print(_('Warning: The proxy does not apply to the requests sent to the lobby!\n'))

		session_data = session.getSessionData()
		if 'proxy' not in session_data or session_data['proxy']['set'] is False:
			print(_('Right now, there is no proxy configured.'))
			proxy_dict = read_proxy()
			if proxy_dict is None:
				event.set()
				return
			session_data['proxy'] = {}
			session_data['proxy']['conf'] = proxy_dict
			session_data['proxy']['set']  = True
		else:
			curr_proxy = session_data['proxy']['conf']['https']
			print(_('Current proxy: {}').format(curr_proxy))
			print(_('What do you want to do?'))
			print(_('0) Exit'))
			print(_('1) Set a new proxy'))
			print(_('2) Remove the current proxy'))
			rta = read(min=0, max=2)

			if rta == 0:
				event.set()
				return
			if rta == 1:
				proxy_dict = read_proxy()
				if proxy_dict is None:
					event.set()
					return
				session_data['proxy']['conf'] = proxy_dict
				session_data['proxy']['set']  = True
			if rta == 2:
				session_data['proxy']['conf'] = {}
				session_data['proxy']['set']  = False
				print(_('The proxy has been removed.'))
				enter()

		session.setSessionData(session_data)
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCity

t = gettext.translation('vacationMode',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def activateVacationMode(session):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	"""
	html = session.get()
	city = getCity(html)

	data = {'action': 'Options', 'function': 'activateVacationMode', 'actionRequest': actionRequest, 'backgroundView': 'city', 'currentCityId': city['id'], 'templateView': 'options_umod_confirm'}
	session.post(params=data, ignoreExpire=True)

def vacationMode(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()
		print(_('Activate vacation mode? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			event.set()
			return

		activateVacationMode(session)

		print(_('Vacation mode has been activated.'))
		enter()
		event.set()
		clear()
		sys.exit()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.getJson import getCity
from ikabot.helpers.signals import setInfoSignal

t = gettext.translation('investigate',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def get_studies(session):
	html = session.get()
	city = getCity(html)
	city_id = city['id']
	url = 'view=researchAdvisor&oldView=updateGlobalData&cityId={0}&backgroundView=city&currentCityId={0}&templateView=researchAdvisor&actionRequest={1}&ajax=1'.format(city_id, actionRequest)
	resp = session.post(url)
	resp = json.loads(resp, strict=False)
	return resp[2][1]

def study(session, studies, num_study):
	html = session.get()
	city = getCity(html)
	city_id = city['id']
	research_type = studies['js_researchAdvisorChangeResearchType{}'.format(num_study)]['ajaxrequest'].split('=')[-1]
	url = 'action=Advisor&function=doResearch&actionRequest={}&type={}&backgroundView=city&currentCityId={}&templateView=researchAdvisor&ajax=1'.format(actionRequest, research_type, city_id)
	session.post(url)

def investigate(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		banner()

		studies = get_studies(session)
		keys = list(studies.keys())
		num_studies = len( [ key for key in keys if 'js_researchAdvisorChangeResearchTypeTxt' in key ] )

		available = []
		for num_study in range(num_studies):
			if 'js_researchAdvisorProgressTxt{}'.format(num_study) in studies:
				available.append(num_study)

		if len(available) == 0:
			print(_('There are no available studies.'))
			enter()
			event.set()
			return

		print(_('Which one do you wish to study?'))
		print('0) None')
		for index, num_study in enumerate(available):
			print('{:d}) {}'.format(index+1, studies['js_researchAdvisorNextResearchName{}'.format(num_study)]))
		choice = read(min=0, max=len(available))

		if choice == 0:
			event.set()
			return

		study(session, studies, available[choice-1])
		print(_('Done.'))
		enter()
		event.set()

	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from decimal import *
from ikabot.config import *

getcontext().prec = 30

def getAvailableResources(html, num=False):
	"""
	Parameters
	----------
	html : string

	Returns
	-------
	resources_available : list[int] | list[str]
	"""
	resources = re.search(r'\\"resource\\":(\d+),\\"2\\":(\d+),\\"1\\":(\d+),\\"4\\":(\d+),\\"3\\":(\d+)}', html)
	if num:
		return [int(resources.group(1)), int(resources.group(3)), int(resources.group(2)), int(resources.group(5)), int(resources.group(4))]
	else:
		return [resources.group(1), resources.group(3), resources.group(2), resources.group(5), resources.group(4)]

def getWarehouseCapacity(html):
	"""
	Parameters
	----------
	html : string
	Returns
	-------
	capacity : int
	"""
	capacity = re.search(r'maxResources:\s*JSON\.parse\(\'{\\"resource\\":(\d+),', html).group(1)
	return int(capacity)

def getWineConsumption(html):
	"""
	Parameters
	----------
	html : string
	Returns
	-------
	capacity : int
	"""
	result = re.search(r'GlobalMenu_WineConsumption"\s*class="rightText">\s*(\d+)\s', html)
	if result:
		return int(result.group(1))
	return 0

def getProductionPerSecond(session, city_id):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	city_id : int

	Returns
	-------
	production: tuple[Decimal, Decimal, int]
	"""
	prod = session.post(payloadPost={'action': 'header', 'function': 'changeCurrentCity', 'actionRequest': actionRequest, 'cityId': city_id, 'ajax': '1'})
	prod = json.loads(prod, strict=False)
	prod = prod[0][1]['headerData']
	wood_production = Decimal( prod['resourceProduction'] )
	luxury_production = Decimal( prod['tradegoodProduction'] )
	luxury_resource_type = int( prod['producedTradegood'] )

	return wood_production, luxury_production, luxury_resource_type



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from ikabot.helpers.resources import *

def getFreeCitizens(html):
	"""This function is used in the ``getCity`` function to determine the amount of free (idle) citizens in the given city.
	Parameters
	----------
	html : str
		a string representing html which is returned when sending a get request to view a city.

	Returns
	-------
	freeCitizens : int
		an integer representing the amount of free citizens in the given city.
	"""
	ciudadanosDisp = re.search(r'js_GlobalMenu_citizens">(.*?)</span>', html).group(1)
	return int(ciudadanosDisp.replace(',', '').replace('.', ''))

def onSale(html):
	"""This function is used in the ``getCity`` function to determine the amount of each resource which is on sale in the branch office
	Parameters
	----------
	html : str
		a string representing html which is returned when sending a get request to view a city.

	Returns
	-------
	onSale : list[int]
		a list containing 5 integers each of which representing the amount of that particular resource which is on sale in the given city. For more information about the order of the resources, refer to ``config.py``
	"""
	rta = re.search(r'branchOfficeResources: JSON\.parse\(\'{\\"resource\\":\\"(\d+)\\",\\"1\\":\\"(\d+)\\",\\"2\\":\\"(\d+)\\",\\"3\\":\\"(\d+)\\",\\"4\\":\\"(\d+)\\"}\'\)', html)
	if rta:
		return [ int(rta.group(1)), int(rta.group(2)), int(rta.group(3)), int(rta.group(4)), int(rta.group(5))]
	else:
		return [0, 0, 0, 0, 0]

def getIsland(html):
	"""This function uses the html passed to it as a string to extract, parse and return an Island object
	Parameters
	----------
	html : str
		the html returned when a get request to view the island is made. This request can be made with the following statement: ``s.get(urlIsla + islandId)``, where ``urlIsla`` is a string defined in ``config.py`` and ``islandId`` is the id of the island.

	Returns
	-------
	island : Island
		this function returns a json parsed Island object. For more information about this object refer to the github wiki page of Ikabot.
	"""
	isla = re.search(r'\[\["updateBackgroundData",([\s\S]*?),"specialServerBadges', html).group(1) + '}'

	isla = isla.replace('buildplace', 'empty')
	isla = isla.replace('xCoord', 'x')
	isla = isla.replace('yCoord', 'y')
	isla = isla.replace(',"owner', ',"')


	# {"id":idIsla,"name":nombreIsla,"x":,"y":,"good":numeroBien,"woodLv":,"goodLv":,"wonder":numeroWonder, "wonderName": "nombreDelMilagro","wonderLv":"5","cities":[{"type":"city","name":cityName,"id":cityId,"level":lvIntendencia,"Id":playerId,"Name":playerName,"AllyId":,"AllyTag":,"state":"vacation"},...}}
	isla = json.loads(isla, strict=False)
	isla['tipo'] = re.search(r'"tradegood":"(\d)"', html).group(1)
	isla['x'] = int(isla['x'])
	isla['y'] = int(isla['y'])

	return isla

def getCity(html):
	"""This function uses the ``html`` passed to it as a string to extract, parse and return a City object
	Parameters
	----------
	html : str
		the html returned when a get request to view the city is made. This request can be made with the following statement: ``s.get(urlCiudad + id)``, where urlCiudad is a string defined in ``config.py`` and id is the id of the city.

	Returns
	-------
	city : dict
		this function returns a json parsed City object. For more information about this object refer to the github wiki page of Ikabot.
	"""

	city = re.search(r'"updateBackgroundData",\s?([\s\S]*?)\],\["updateTemplateData"', html).group(1)
	city = json.loads(city, strict=False)

	city['Id'] = city.pop('ownerId')
	city['Name'] = city.pop('ownerName')
	city['x'] = int(city.pop('islandXCoord'))
	city['y'] = int(city.pop('islandYCoord'))
	city['cityName'] = city['name']

	i = 0
	for position in city['position']:
		position['position'] = i
		i += 1
		if 'level' in position:
			position['level'] = int(position['level'])
		position['isBusy'] = False
		if 'constructionSite' in position['building']:
			position['isBusy'] = True
			position['building'] = position['building'][:-17]
		elif 'buildingGround ' in position['building']:
			position['name'] = 'empty'
			position['type'] = position['building'].split(' ')[-1]
			position['building'] = 'empty'

	city['id'] = str(city['id'])
	city['propia'] = True
	city['recursos'] = getAvailableResources(html, num=True)
	city['storageCapacity'] = getWarehouseCapacity(html)
	city['ciudadanosDisp'] = getFreeCitizens(html)
	city['consumo'] = getWineConsumption(html)
	city['enventa'] = onSale(html)
	city['freeSpaceForResources'] = []
	for i in range(5):
		city['freeSpaceForResources'].append( city['storageCapacity'] - city['recursos'][i] - city['enventa'][i] )

	return city



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.process import run
from ikabot.helpers.gui import *
from ikabot.config import *

t = gettext.translation('update',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def update(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		print(_('To update ikabot run:'))
		print('python3 -m pip install --user --upgrade ikabot')
		enter()
		event.set()
	except KeyboardInterrupt:
		event.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import sys
import json
import gettext
import os
from ikabot import config
from decimal import *
from ikabot.config import *
from ikabot.helpers.getJson import *
from ikabot.helpers.gui import *

t = gettext.translation('pedirInfo',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def read(min=None, max=None, digit=False, msg=prompt, values=None, empty=False, additionalValues=None, default=None): # user input
	"""Reads input from user
	Parameters
	----------
	min : int
		smallest number acceptable as input
	max : int
		greatest number acceptable as input
	digit : bool
		boolean indicating whether or not the input MUST be an int
	msg : str
		string printed before the user is asked for input
	values : list
		list of strings which are acceptable as input
	empty : bool
		a boolean indicating whether or not an empty string is acceptable as input
	additionalValues : list
		list of strings which are additional valid inputs. Can be used with digit = True to validate a string as an input among all digits

	Returns
	-------
	result : int | str
		int representing the user's choice
	"""
	try:
		if len(config.predetermined_input) !=0:
			return config.predetermined_input.pop(0)
	except Exception:
		pass

	def _invalid():
		print('\033[1A\033[K', end="") # remove line
		return read(min, max, digit, msg, values, additionalValues = additionalValues)

	try:
		read_input = input(msg)
	except EOFError:
		return _invalid()

	if additionalValues is not None and read_input in additionalValues:
		return read_input

	if read_input == '' and default is not None:
		return default

	if read_input == '' and empty is True:
		return read_input

	if digit is True or min is not None or max is not None:
		if read_input.isdigit() is False:
			return _invalid()
		else:
			try:
				read_input = eval(read_input)
			except SyntaxError:
				return _invalid()
	if min is not None and read_input < min:
		return _invalid()
	if max is not None and read_input > max:
		return _invalid()
	if values is not None and read_input not in values:
		return _invalid()
	return read_input

def chooseCity(session, foreign=False):
	"""Prompts the user to chose a city
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	foreign : bool
		lets the user choose a foreign city

	Returns
	-------
	city : City
		a city object representing the chosen city
	"""
	global menu_cities
	(ids, cities) = getIdsOfCities(session)
	if menu_cities == '':
		longest_city_name_length = 0
		for city_id in ids:
			length = len(cities[city_id]['name'])
			if length > longest_city_name_length:
				longest_city_name_length = length
		pad = lambda city_name: ' ' * (longest_city_name_length - len(city_name) + 2)
		resources_abbreviations = {'1': _('(W)'), '2': _('(M)'), '3': _('(C)'), '4': _('(S)')}

		i = 0
		if foreign:
			print(_(' 0: foreign city'))
		else:
			print('')
		for city_id in ids:
			i += 1
			resource_index = cities[city_id]['tradegood']
			resource_abb = resources_abbreviations[resource_index]
			city_name = cities[city_id]['name']
			matches = re.findall(r'u[0-9a-f]{4}', city_name)
			for match in matches:
				to_unicode = '\\' + match
				to_unicode = to_unicode.encode().decode('unicode-escape')
				city_name = city_name.replace(match, to_unicode)
			num = ' ' + str(i) if i < 10 else str(i)
			menu_cities += '{}: {}{}{}\n'.format(num, city_name, pad(city_name), resource_abb)
		menu_cities = menu_cities[:-1]
	if foreign:
		print(_(' 0: foreign city'))
	print(menu_cities)

	if foreign:
		selected_city_index = read(min=0, max=len(ids))
	else:
		selected_city_index = read(min=1, max=len(ids))
	if selected_city_index == 0:
		return chooseForeignCity(session)
	else:
		html = session.get(city_url + ids[selected_city_index - 1])
		return getCity(html)

def chooseForeignCity(session):
	"""Prompts the user to select an island, and a city on that island (is only used in chooseCity)
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object

	Returns
	-------
	 city : City
		a city object representing the city the user chose
	"""
	banner()
	x = read(msg='coordinate x:', digit=True)
	y = read(msg='coordinate y:', digit=True)
	print('')
	url = 'view=worldmap_iso&islandX={}&islandY={}&oldBackgroundView=island&islandWorldviewScale=1'.format(x, y)
	html = session.get(url)
	try:
		islands_json = re.search(r'jsonData = \'(.*?)\';', html).group(1)
		islands_json = json.loads(islands_json, strict=False)
		island_id = islands_json['data'][str(x)][str(y)][0]
	except:
		print(_('Incorrect coordinates'))
		enter()
		banner()
		return chooseCity(session, foreign=True)
	html = session.get(island_url + island_id)
	island = getIsland(html)
	longest_city_name_length = 0
	for city in island['cities']:
		if city['type'] == 'city':
			city_name_length = len(city['name'])
			if city_name_length > longest_city_name_length:
				longest_city_name_length = city_name_length
	pad = lambda name: ' ' * (longest_city_name_length - len(name) + 2)
	i = 0
	city_options = []
	for city in island['cities']:
		if city['type'] == 'city' and city['state'] == '' and city['Name'] != session.username:
			i += 1
			num = ' ' + str(i) if i < 10 else str(i)
			print('{}: {}{}({})'.format(num, city['name'], pad(city['name']), city['Name']))
			city_options.append(city)
	if i == 0:
		print(_('There are no cities where to send resources on this island'))
		enter()
		return chooseCity(session, foreign=True)
	selected_city_index = read(min=1, max=i)
	city = city_options[selected_city_index - 1]
	city['islandId'] = island['id']
	city['cityName'] = city['name']
	city['propia'] = False
	return city

def askForValue(text, max):
	"""Displays text and asks the user to enter a value between 0 and max

	Parameters
	----------
	text : str
		text to be displayed when asking the user for input
	max : int
		integer representing the number of input options

	Returns
	-------
	var : int
		integer representing the user's input
		if the user has inputed nothing, 0 will be returned instead
	"""
	var = read(msg=text, min=0, max=max, empty=True)
	if var == '':
		var = 0
	return var

def getIdsOfCities(session, all=False):
	"""Gets the user's cities
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object
	all : bool
		boolean indicating whether all cities should be returned, or only those that belong to the current user

	Returns
	-------
	(ids, cities) : tuple
		a tuple containing the a list of city IDs and a list of city objects
	"""
	global cities_cache
	global ids_cache
	if ids_cache is None or cities_cache is None or session.padre is False:
		html = session.get()
		cities_cache = re.search(r'relatedCityData:\sJSON\.parse\(\'(.+?),\\"additionalInfo', html).group(1) + '}'
		cities_cache = cities_cache.replace('\\', '')
		cities_cache = cities_cache.replace('city_', '')
		cities_cache = json.loads(cities_cache, strict=False)

		ids_cache = [city_id for city_id in cities_cache]
		ids_cache = sorted(ids_cache)

	# {'coords': '[x:y] ', 'id': idCiudad, 'tradegood': '..', 'name': 'nomberCiudad', 'relationship': 'ownCity'|'occupiedCities'|..}
	if all is False:
		ids_own   = [city_id for city_id in cities_cache if cities_cache[city_id]['relationship'] == 'ownCity']
		ids_other = [city_id for city_id in cities_cache if cities_cache[city_id]['relationship'] != 'ownCity']
		own_cities = cities_cache.copy()
		for id in ids_other:
			del own_cities[id]
		return ids_own, own_cities
	else:
		return ids_cache, cities_cache

def getIslandsIds(session):
	"""Gets the IDs of islands the user has cities on
	Parameters
	----------
	session : ikabot.web.session.Session
		Session object

	Returns
	-------
	islands_ids : list
		a list containing the IDs of the users islands
	"""
	(cities_ids, cities) = getIdsOfCities(session)
	islands_ids = set()
	for city_id in cities_ids:
		html = session.get(city_url + city_id)
		city = getCity(html)
		island_id = city['islandId']
		islands_ids.add(island_id)
	return list(islands_ids)



import setuptools

setuptools.setup(
	name="ikabot",
	version="6.5.3",
	author="physics-sp",
	author_email="physics-sp@protonmail.com",
	license='MIT',
	description="A bot for ikariam",
	url="https://github.com/physics-sp/ikabot",
	include_package_data=True,
	packages=setuptools.find_packages(),
	install_requires=[
		  'requests',
		  'requests[socks]',
		  'cryptography',
		  'psutil'
	],
	entry_points = {
		'console_scripts': ['ikabot=ikabot.command_line:main'],
	},
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License"
        ],
)




#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import gettext
import sys
import datetime
from ikabot.helpers.pedirInfo import read, enter
from ikabot.helpers.gui import *
from ikabot.config import *
from ikabot.helpers.process import updateProcessList, run

t = gettext.translation('killTasks',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext

def killTasks(session, event, stdin_fd, predetermined_input):
	"""
	Parameters
	----------
	session : ikabot.web.session.Session
	event : multiprocessing.Event
	stdin_fd: int
	predetermined_input : multiprocessing.managers.SyncManager.list
	"""
	sys.stdin = os.fdopen(stdin_fd)
	config.predetermined_input = predetermined_input
	try:
		while (True) :
			banner()
			process_list = updateProcessList(session)
			print('Which task do you wish to kill?')
			print()
			print('(0) Exit')
			for process in process_list:
				if 'date' in process:
					print("({}) {:<35}{:>20}".format(process_list.index(process) + 1, process['action'], datetime.datetime.fromtimestamp(process['date']).strftime('%b %d %H:%M:%S')))
				else:
					print("({}) {:<35}".format(process_list.index(process) + 1, process['action'],))
			action = read(min=0, max=len(process_list), digit=True)
			if action == 0:
				event.set()
				return
			elif process_list[action-1]['action'] == 'killTasks':
				print('I will now commit suicide 😂🔫')
				enter()
				event.set()
				return
			else:
				if isWindows:
					run("taskkill /F /PID {}".format(process_list[action-1]['pid']))
				else:
					run("kill -9 {}".format(process_list[action-1]['pid']))
		
		
	except KeyboardInterrupt:
		event.set()
		return


#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import sys
import json
import time
import random
import getpass
import datetime
import gettext
import requests
import base64
from ikabot import config
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import banner
from ikabot.helpers.aesCipher import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCity

t = gettext.translation('session',
                        localedir,
                        languages=languages,
                        fallback=True)
_ = t.gettext


class Session:
	def __init__(self):
		if isWindows:
			self.logfile = os.getenv('temp') + '/ikabot.log'
		else:
			self.logfile = '/tmp/ikabot.log'
		self.log = False
		self.padre = True
		self.logged = False
		self.__login()

	def writeLog(self, msg):
		return self.__log(msg)

	def __log(self, msg):
		if self.log is False:
			return
		now = datetime.datetime.now()
		entry = '{}.{:02}.{:02} {:02d}:{:02}:{:02}\t{:d}: {}\n'.format(now.year, now.month, now.day, now.hour, now.minute, now.second, os.getpid(), msg)
		fh = open(self.logfile, 'a')
		fh.write(entry)
		fh.close()

	def __genRand(self):
		return hex(random.randint(0, 65535))[2:]

	def __genCookie(self):
		return self.__genRand() + self.__genRand() + hex(int(round(time.time() * 1000)))[2:] + self.__genRand() + self.__genRand()

	def __fp_eval_id(self):
		return self.__genRand() + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + self.__genRand() + self.__genRand()

	def __logout(self, html):
		if html is not None:
			idCiudad = getCity(html)['id']
			token = re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)
			urlLogout = 'action=logoutAvatar&function=logout&sideBarExt=undefined&startPageShown=1&detectedDevice=1&cityId={0}&backgroundView=city&currentCityId={0}&actionRequest={1}'.format(idCiudad, token)
			self.s.get(self.urlBase + urlLogout)

	def __isInVacation(self, html):
		return 'nologin_umod' in html

	def __isExpired(self, html):
		return 'index.php?logout' in html
	
	def isExpired(self,html):
		return self.__isExpired(html)

	def __saveNewCookies(self):
		sessionData = self.getSessionData()

		cookie_dict = dict(self.s.cookies.items())
		sessionData['cookies'] = cookie_dict

		self.setSessionData(sessionData)

	def __getCookie(self, sessionData=None):
		if sessionData is None:
			sessionData = self.getSessionData()
		try:
			cookie_dict = sessionData['cookies']
			self.s = requests.Session()
			self.__update_proxy(sessionData=sessionData)
			self.s.headers.clear()
			self.s.headers.update(self.headers)
			requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
		except KeyError:
			self.__login(3)

	def __login(self, retries=0):
		self.__log('__login()')
		if not self.logged:
			banner()

			self.mail = read(msg=_('Mail:'))

			if len(config.predetermined_input) != 0:
				self.password = config.predetermined_input.pop(0)
			else:
				self.password = getpass.getpass(_('Password:'))

			banner()

		self.s = requests.Session()

		# get gameEnvironmentId and platformGameId
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/config/configuration.js')

		js = r.text
		gameEnvironmentId = re.search(r'"gameEnvironmentId":"(.*?)"', js)
		if gameEnvironmentId is None:
			sys.exit('gameEnvironmentId not found')
		gameEnvironmentId = gameEnvironmentId.group(1)
		platformGameId = re.search(r'"platformGameId":"(.*?)"', js)
		if platformGameId is None:
			sys.exit('platformGameId not found')
		platformGameId = platformGameId.group(1)

		# get __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/js/connect.js')
		html = r.text
		captcha = re.search(r'Attention Required', html)
		if captcha is not None:
			sys.exit('Captcha error!')

		# update __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/config')

		__fp_eval_id_1 = self.__fp_eval_id()
		__fp_eval_id_2 = self.__fp_eval_id()
		try:
			# get pc_idt cookie
			self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
			self.s.headers.clear()
			self.s.headers.update(self.headers)
			data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'VISIT', 'replacement_kid': '', 'fp_eval_id': __fp_eval_id_1, 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fingerprint': '2175408712', 'fp_exec_time': '1.00'}
			r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

			# update pc_idt cookie
			self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
			self.s.headers.clear()
			self.s.headers.update(self.headers)
			data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'fp_eval', 'fp_eval_id': __fp_eval_id_2, 'fingerprint': '2175408712', 'fp2_config_id': '1', 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fp2_value': '921af958be7cf2f76db1e448c8a5d89d', 'fp2_exec_time': '96.00'}
			r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)
		except Exception:
			pass # These cookies are not required and sometimes cause issues for people logging in

		# options req (not really needed)
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate','Access-Control-Request-Method': 'POST','Access-Control-Request-Headers': 'content-type,tnt-installation-id','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.options('https://gameforge.com/api/v1/auth/thin/sessions')

		# send creds
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate, br','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','TNT-Installation-Id': '','Content-Type': 'application/json','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'keep-alive','Pragma': 'no-cache','Cache-Control': 'no-cache','TE': 'Trailers'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {"identity": self.mail, "password": self.password, "locale":"es_AR", "gfLang":"ar", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": "false"}
		r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)
		if 'gf-challenge-id' in r.headers:
			sys.exit(_('Captcha error! (Interactive)'))
		if r.status_code == 403:
			sys.exit(_('Wrong email or password\n'))

		# get the authentication token and set the cookie
		ses_json = json.loads(r.text, strict=False)
		auth_token = ses_json['token']
		cookie_obj = requests.cookies.create_cookie(domain='.gameforge.com', name='gf-token-production', value=auth_token)
		self.s.cookies.set_cookie(cookie_obj)

		# get accounts
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/accounts')
		accounts = json.loads(r.text, strict=False)

		# get servers
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/servers')
		servers = json.loads(r.text, strict=False)

		if not self.logged:

			if len([ account for account in accounts if account['blocked'] is False ]) == 1:
				self.account  = [ account for account in accounts if account['blocked'] is False ][0]
			else:
				print(_('With which account do you want to log in?\n'))

				max_name = max( [ len(account['name']) for account in accounts if account['blocked'] is False ] )
				i = 0
				for account in [ account for account in accounts if account['blocked'] is False ]:
					server = account['server']['language']
					mundo = account['server']['number']
					world = [ srv['name'] for srv in servers if srv['language'] == server and srv['number'] == mundo ][0]
					i += 1
					pad = ' ' * (max_name - len(account['name']))
					print('({:d}) {}{} [{} - {}]'.format(i, account['name'], pad, server, world))
				num = read(min=1, max=i)
				self.account  = [ account for account in accounts if account['blocked'] is False ][num - 1]
			self.username = self.account['name']
			self.servidor = self.account['server']['language']
			self.mundo    = str(self.account['server']['number'])
			self.word     = [ srv['name'] for srv in servers if srv['language'] == self.servidor and srv['number'] == int(self.mundo) ][0]
			config.infoUser = _('Server:{}').format(self.servidor)
			config.infoUser += _(', World:{}').format(self.word)
			config.infoUser += _(', Player:{}').format(self.username)
			banner()


		self.host = 's{}-{}.ikariam.gameforge.com'.format(self.mundo, self.servidor)
		self.urlBase = 'https://{}/index.php?'.format(self.host)

		self.headers = {'Host': self.host, 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 'Referer': 'https://{}'.format(self.host), 'X-Requested-With': 'XMLHttpRequest', 'Origin': 'https://{}'.format(self.host), 'DNT': '1', 'Connection': 'keep-alive', 'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}

		self.cipher = AESCipher(self.mail, self.username, self.password)
		sessionData = self.getSessionData()

		used_old_cookies = False
		# if there are cookies stored, try to use them
		if 'cookies' in sessionData and self.logged is False:
			# create a new temporary session object
			old_s = requests.Session()
			# set the headers
			old_s.headers.clear()
			old_s.headers.update(self.headers)
			# set the cookies to test
			cookie_dict = sessionData['cookies']
			requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=old_s.cookies, overwrite=True)
			self.__update_proxy(obj=old_s, sessionData=sessionData)
			try:
				# make a request to check the connection
				html = old_s.get(self.urlBase).text
			except:
				self.__proxy_error()

			cookies_are_valid = self.__isExpired(html) is False
			if cookies_are_valid:
				self.__log('using old cookies')
				used_old_cookies = True
				# assign the old cookies to the session object
				requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
				# set the proxy
				self.__update_proxy(sessionData=sessionData)
				# set the headers
				self.s.headers.clear()
				self.s.headers.update(self.headers)

		# login as normal and get new cookies
		if used_old_cookies is False:
			self.__log('using new cookies')
			resp = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/loginLink?id={}&server[language]={}&server[number]={}'.format(self.account['id'], self.servidor, self.mundo)).text
			resp = json.loads(resp, strict=False)
			if 'url' not in resp:
				if retries > 0:
					return self.__login(retries-1)
				else:
					msg = 'Login Error: ' + str(resp)
					if self.padre:
						print(msg)
						sys.exit()
					else:
						sys.exit(msg)

			url = resp['url']
			match = re.search(r'https://s\d+-\w{2}\.ikariam\.gameforge\.com/index\.php\?', url)
			if match is None:
				sys.exit('Error')

			# set the headers
			self.s.headers.clear()
			self.s.headers.update(self.headers)

			# set the proxy
			self.__update_proxy(sessionData=sessionData)

			# use the new cookies instead, invalidate the old ones
			try:
				html = self.s.get(url).text
			except:
				self.__proxy_error()

		if self.__isInVacation(html):
			msg = _('The account went into vacation mode')
			if self.padre:
				print(msg)
			else:
				sendToBot(self, msg)
			os._exit(0)
		if self.__isExpired(html):
			if retries > 0:
				return self.__login(retries-1)
			if self.padre:
				msg = _('Login error.')
				print(msg)
				os._exit(0)
			raise Exception('Couldn\'t log in')

		if not used_old_cookies:
			self.__saveNewCookies()

		self.logged = True

	def __backoff(self):
		self.__log('__backoff()')
		if self.padre is False:
			time.sleep(5 * random.randint(0, 10))

	def __sessionExpired(self):
		self.__log('__sessionExpired()')
		self.__backoff()

		sessionData = self.getSessionData()

		try:
			if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
				self.__getCookie(sessionData)
			else:
				try:
					self.__login(3)
				except Exception:
					self.__sessionExpired()
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def __proxy_error(self):
		sessionData = self.getSessionData()
		if 'proxy' not in sessionData or sessionData['proxy']['set'] is False:
			sys.exit('network error')
		if self.padre is True:
			print(_('There seems to be a problem connecting to ikariam.'))
			print(_('Do you want to disable the proxy? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				sys.exit()
			else:
				sessionData['proxy'] = {}
				sessionData['proxy']['conf'] = {}
				sessionData['proxy']['set']  = False
				self.setSessionData(sessionData)
				print(_('Proxy disabled, try again.'))
				enter()
				sys.exit()
		else:
			msg = _('Network error. Consider disabling the proxy.')
			sendToBot(self, msg)
			sys.exit()

	def __update_proxy(self, *, obj=None, sessionData=None):
		# set the proxy
		if obj is None:
			obj = self.s
		if sessionData is None:
			sessionData = self.getSessionData()
		if 'proxy' in sessionData:
			obj.proxies.update(sessionData['proxy']['conf'])

	def __checkCookie(self):
		self.__log('__checkCookie()')
		sessionData = self.getSessionData()

		try:
			if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
				self.__getCookie(sessionData)
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def __token(self):
		"""Generates a valid actionRequest token from the session
		Returns
		-------
		token : str
			a string representing a valid actionRequest token
		"""
		html = self.get()
		return re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)

	def get(self, url='', params={}, ignoreExpire=False, noIndex=False, fullResponse=False):
		"""Sends get request to ikariam
		Parameters
		----------
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the get request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the get request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url params and sending the get request

		Returns
		-------
		html : str
			response from the server
		"""
		self.__checkCookie()
		self.__update_proxy()

		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		self.__log('get({}), params:{}'.format(url, str(params)))
		while True:
			try:
				if fullResponse:
					response = self.s.get(url, params=params)
				else:
					response = self.s.get(url, params=params).text #this isn't recursion, this get is different from the one it's in
				html = response
				if ignoreExpire is False:
					assert self.__isExpired(html) is False
				return html
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def post(self, url='', payloadPost={}, params={}, ignoreExpire=False, noIndex=False):
		"""Sends post request to ikariam
		Parameters
		----------
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		payloadPost : dict
			dictionary containing key-value pairs which represent the payload of the post request
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the post request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the post request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url and params and sending the post request

		Returns
		-------
		html : str
			response from the server
		"""
		url_original = url
		payloadPost_original = payloadPost
		params_original = params
		self.__checkCookie()
		self.__update_proxy()
		
		# add the request id
		token = self.__token()
		url = url.replace(actionRequest, token)
		if 'actionRequest' in payloadPost:
			payloadPost['actionRequest'] = token
		if 'actionRequest' in params:
			params['actionRequest'] = token

		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		self.__log('post({}), data={}'.format(url, str(payloadPost)))
		while True:
			try:
				resp = self.s.post(url, data=payloadPost, params=params).text
				if ignoreExpire is False:
					assert self.__isExpired(resp) is False
				if 'TXT_ERROR_WRONG_REQUEST_ID' in resp:
					self.__log(_('got TXT_ERROR_WRONG_REQUEST_ID'))
					return self.post(url=url_original, payloadPost=payloadPost_original, params=params_original, ignoreExpire=ignoreExpire, noIndex=noIndex)
				self.__log(resp)
				return resp
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def logout(self):
		"""This function kills the current (chlid) process
		"""
		self.__log('logout({})')
		if self.padre is False:
			os._exit(0)

	def setSessionData(self, sessionData):
		"""Encrypts relevant session data and writes it to the .ikabot file
		Parameters
		----------
		sessionData : dict
			dictionary containing relevant session data, data is written to file using AESCipher.setSessionData
		"""
		self.cipher.setSessionData(self, sessionData)

	def getSessionData(self):
		"""Gets relevant session data from the .ikabot file
		"""
		return self.cipher.getSessionData(self)

def normal_get(url, params={}):
	"""Sends a get request to provided url
	Parameters
	----------
	url : str
		a string representing the url to which to send the get request
	params : dict
		a dictionary containing key-value pairs which represent the parameters of the get request

	Returns
	-------
	response : requests.Response
		a requests.Response object which represents the webservers response. For more information on requests.Response refer to https://requests.readthedocs.io/en/master/api/#requests.Response
	"""
	try:

		return requests.get(url, params=params)

	except requests.exceptions.ConnectionError:
		sys.exit(_('Internet connection failed'))
